@@ -1,6 +1,6 @@
-/* pkclist.c
+/* pkclist.c - create a list of public keys
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
- *               2008, 2010 Free Software Foundation, Inc.
+ *               2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -22,17 +22,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <errno.h>
 #include <assert.h>
 
+#include "gpg.h"
 #include "options.h"
 #include "packet.h"
-#include "errors.h"
+#include "status.h"
 #include "keydb.h"
-#include "memory.h"
 #include "util.h"
 #include "main.h"
 #include "trustdb.h"
@@ -74,10 +71,10 @@ do_show_revocation_reason( PKT_signature *sig )
 
 	log_info( _("reason for revocation: ") );
 	if( text )
-	    fputs( text, log_stream() );
+	    fputs( text, log_get_stream() );
 	else
-	    fprintf( log_stream(), "code=%02x", *p );
-	putc( '\n', log_stream() );
+	    fprintf( log_get_stream(), "code=%02x", *p );
+	log_printf ("\n");
 	n--; p++;
 	pp = NULL;
 	do {
@@ -89,9 +86,9 @@ do_show_revocation_reason( PKT_signature *sig )
 	    if( n ) {
 		pp = memchr( p, '\n', n );
 		nn = pp? pp - p : n;
-		log_info( _("revocation comment: ") );
-		print_string( log_stream(), p, nn, 0 );
-		putc( '\n', log_stream() );
+		log_info ( _("revocation comment: ") );
+		print_string ( log_get_stream(), p, nn, 0 );
+		log_printf ("\n");
 		p += nn; n -= nn;
 	    }
 	} while( pp );
@@ -162,7 +159,7 @@ show_revocation_reason( PKT_public_key *pk, int mode )
  * mode: 0 = standard
  *       1 = Without key info and additional menu option 'm'
  *           this does also add an option to set the key to ultimately trusted.
- * Returns:
+ * Returns: 
  *      -2 = nothing changed - caller should show some additional info
  *      -1 = quit operation
  *       0 = nothing changed
@@ -194,11 +191,11 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
   for(;;) {
     /* A string with valid answers.
 
-       TRANSLATORS: These are the allowed answers in lower and
+       Note to translators: These are the allowed answers in lower and
        uppercase.  Below you will find the matching strings which
        should be translated accordingly and the letter changed to
        match the one in the answer string.
-
+    
          i = please show me more information
          m = back to the main menu
          s = skip this key
@@ -206,9 +203,9 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
     */
     const char *ans = _("iImMqQsS");
 
-    if( !did_help )
+    if( !did_help ) 
       {
-        if( !mode )
+        if( !mode ) 
           {
             KBNODE keyblock, un;
 
@@ -235,7 +232,7 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
                 if (un->pkt->pkt.user_id->is_primary
 		    && !un->pkt->pkt.user_id->attrib_data )
 		  continue;
-
+                
 		if((opt.verify_options&VERIFY_SHOW_PHOTOS)
 		   && un->pkt->pkt.user_id->attrib_data)
 		  show_photos(un->pkt->pkt.user_id->attribs,
@@ -247,7 +244,7 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
 
 		tty_printf(_("  aka \"%s\"\n"),p);
 	      }
-
+        
             print_fingerprint (pk, NULL, 2);
             tty_printf("\n");
 	    release_kbnode (keyblock);
@@ -305,7 +302,7 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
       did_help = 0;
     else if( *p && p[1] )
       ;
-    else if( !p[1] && ((*p >= '0'+min_num) && *p <= (mode?'5':'4')) )
+    else if( !p[1] && ((*p >= '0'+min_num) && *p <= (mode?'5':'4')) ) 
       {
         unsigned int trust;
         switch( *p )
@@ -331,14 +328,14 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
       }
 #if 0
     /* not yet implemented */
-    else if( *p == ans[0] || *p == ans[1] )
+    else if( *p == ans[0] || *p == ans[1] ) 
       {
         tty_printf(_("Certificates leading to an ultimately trusted key:\n"));
         show = 1;
         break;
       }
 #endif
-    else if( mode && (*p == ans[2] || *p == ans[3] || *p == CONTROL_D ) )
+    else if( mode && (*p == ans[2] || *p == ans[3] || *p == CONTROL_D ) ) 
       {
         break ; /* back to the menu */
       }
@@ -357,15 +354,15 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
   return show? -2: quit? -1 : changed;
 }
 
-/*
+/* 
  * Display a menu to change the ownertrust of the key PK (which should
- * be a primary key).
+ * be a primary key).  
  * For mode values see do_edit_ownertrust ()
  */
 int
 edit_ownertrust (PKT_public_key *pk, int mode )
 {
-  unsigned int trust = 0;  /* Needs to be initialized to avoid gcc warning. */
+  unsigned int trust = 0;
   int no_help = 0;
 
   for(;;)
@@ -416,7 +413,7 @@ do_we_trust( PKT_public_key *pk, unsigned int trustlevel )
       log_error ("invalid trustlevel %u returned from validation layer\n",
 		 trustlevel);
       /* fall thru */
-    case TRUST_UNKNOWN:
+    case TRUST_UNKNOWN: 
     case TRUST_UNDEFINED:
       log_info(_("%s: There is no assurance this key belongs"
 		 " to the named user\n"),keystr_from_pk(pk));
@@ -466,6 +463,18 @@ do_we_trust_pre( PKT_public_key *pk, unsigned int trustlevel )
 
       tty_printf("\n");
 
+      
+      if (is_status_enabled ())
+        {
+          u32 kid[2];
+          char *hint_str;
+          
+          keyid_from_pk (pk, kid);
+          hint_str = get_long_user_id_string ( kid );
+          write_status_text ( STATUS_USERID_HINT, hint_str );
+          xfree (hint_str);
+        }
+
       if( cpr_get_answer_is_yes("untrusted_key.override",
 				_("Use this key anyway? (y/N) "))  )
 	rc = 1;
@@ -491,7 +500,7 @@ check_signatures_trust( PKT_signature *sig )
   int rc=0;
 
   rc = get_pubkey( pk, sig->keyid );
-  if (rc)
+  if (rc) 
     { /* this should not happen */
       log_error("Ooops; the key vanished  - can't check the trust\n");
       rc = G10ERR_NO_PUBKEY;
@@ -513,7 +522,7 @@ check_signatures_trust( PKT_signature *sig )
 
   trustlevel = get_validity (pk, NULL);
 
-  if ( (trustlevel & TRUST_FLAG_REVOKED) )
+  if ( (trustlevel & TRUST_FLAG_REVOKED) ) 
     {
       write_status( STATUS_KEYREVOKED );
       if(pk->is_revoked==2)
@@ -524,13 +533,13 @@ check_signatures_trust( PKT_signature *sig )
       log_info(_("         This could mean that the signature is forged.\n"));
       show_revocation_reason( pk, 0 );
     }
-  else if ((trustlevel & TRUST_FLAG_SUB_REVOKED) )
+  else if ((trustlevel & TRUST_FLAG_SUB_REVOKED) ) 
     {
       write_status( STATUS_KEYREVOKED );
       log_info(_("WARNING: This subkey has been revoked by its owner!\n"));
       show_revocation_reason( pk, 0 );
     }
-
+  
   if ((trustlevel & TRUST_FLAG_DISABLED))
     log_info (_("Note: This key has been disabled.\n"));
 
@@ -563,9 +572,9 @@ check_signatures_trust( PKT_signature *sig )
                       "does not match DNS entry\n"), sig->pka_info->email);
         }
 
-      switch ( (trustlevel & TRUST_MASK) )
+      switch ( (trustlevel & TRUST_MASK) ) 
         {
-        case TRUST_UNKNOWN:
+        case TRUST_UNKNOWN: 
         case TRUST_UNDEFINED:
         case TRUST_MARGINAL:
           if (okay && opt.verify_options&VERIFY_PKA_TRUST_INCREASE)
@@ -587,18 +596,18 @@ check_signatures_trust( PKT_signature *sig )
     }
 
   /* Now let the user know what up with the trustlevel. */
-  switch ( (trustlevel & TRUST_MASK) )
+  switch ( (trustlevel & TRUST_MASK) ) 
     {
     case TRUST_EXPIRED:
       log_info(_("Note: This key has expired!\n"));
       print_fingerprint (pk, NULL, 1);
       break;
-
+        
     default:
       log_error ("invalid trustlevel %u returned from validation layer\n",
                  trustlevel);
       /* fall thru */
-    case TRUST_UNKNOWN:
+    case TRUST_UNKNOWN: 
     case TRUST_UNDEFINED:
       write_status( STATUS_TRUST_UNDEFINED );
       log_info(_("WARNING: This key is not certified with"
@@ -615,7 +624,7 @@ check_signatures_trust( PKT_signature *sig )
       log_info(_("         The signature is probably a FORGERY.\n"));
       if (opt.with_fingerprint)
         print_fingerprint (pk, NULL, 1);
-      rc = G10ERR_BAD_SIGN;
+      rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
       break;
 
     case TRUST_MARGINAL:
@@ -705,7 +714,7 @@ default_recipient(void)
 }
 
 static int
-expand_id(const char *id,STRLIST *into,unsigned int flags)
+expand_id(const char *id,strlist_t *into,unsigned int flags)
 {
   struct groupitem *groups;
   int count=0;
@@ -715,7 +724,7 @@ expand_id(const char *id,STRLIST *into,unsigned int flags)
       /* need strcasecmp() here, as this should be localized */
       if(strcasecmp(groups->name,id)==0)
 	{
-	  STRLIST each,sl;
+	  strlist_t each,sl;
 
 	  /* this maintains the current utf8-ness */
 	  for(each=groups->values;each;each=each->next)
@@ -734,10 +743,10 @@ expand_id(const char *id,STRLIST *into,unsigned int flags)
 
 /* For simplicity, and to avoid potential loops, we only expand once -
    you can't make an alias that points to an alias. */
-static STRLIST
-expand_group(STRLIST input)
+static strlist_t
+expand_group(strlist_t input)
 {
-  STRLIST sl,output=NULL,rover;
+  strlist_t sl,output=NULL,rover;
 
   for(rover=input;rover;rover=rover->next)
     if(expand_id(rover->d,&output,rover->flags)==0)
@@ -766,20 +775,19 @@ expand_group(STRLIST input)
      Bit 1: This is a hidden recipient.
 
    USE is the desired use for the key - usually PUBKEY_USAGE_ENC.
-   RET_PK_LIST.
 
    On success a list of keys is stored at the address RET_PK_LIST; the
    caller must free this list.  On error the value at this address is
    not changed.
  */
 int
-build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
+build_pk_list( strlist_t rcpts, PK_LIST *ret_pk_list, unsigned int use )
 {
   PK_LIST pk_list = NULL;
   PKT_public_key *pk=NULL;
   int rc=0;
   int any_recipients=0;
-  STRLIST rov,remusr;
+  strlist_t rov,remusr;
   char *def_rec = NULL;
 
   /* Try to expand groups if any have been defined. */
@@ -790,7 +798,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
 
   /* Check whether there are any recipients in the list and build the
    * list of the encrypt-to ones (we always trust them). */
-  for ( rov = remusr; rov; rov = rov->next )
+  for ( rov = remusr; rov; rov = rov->next ) 
     {
       if ( !(rov->flags & 1) )
         {
@@ -809,7 +817,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
               compliance_failure();
             }
         }
-      else if ( (use & PUBKEY_USAGE_ENC) && !opt.no_encrypt_to )
+      else if ( (use & PUBKEY_USAGE_ENC) && !opt.no_encrypt_to ) 
         {
           /* Encryption has been requested and --encrypt-to has not
              been disabled.  Check this encrypt-to key. */
@@ -817,29 +825,25 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
           pk->req_usage = use;
 
           /* We explicitly allow encrypt-to to an disabled key; thus
-             we pass 1 as last argument. */
-          if ( (rc = get_pubkey_byname ( pk, rov->d, NULL, NULL, 1 )) )
+             we pass 1for the second last argument and 1 as the last
+             argument to disable AKL. */
+          if ( (rc = get_pubkey_byname (NULL, pk, rov->d, NULL, NULL, 1, 1)) ) 
             {
               free_public_key ( pk ); pk = NULL;
               log_error (_("%s: skipped: %s\n"), rov->d, g10_errstr(rc) );
-              write_status_text_and_buffer (STATUS_INV_RECP,
-                                            (rc == G10ERR_NO_PUBKEY
-                                             || rc == G10ERR_NO_SECKEY)? "1 ":
-                                            (rc == G10ERR_INV_USER_ID)? "14 ":
-                                            "0 ",
+              write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
                                             rov->d, strlen (rov->d), -1);
               goto fail;
             }
-          else if ( !(rc=check_pubkey_algo2 (pk->pubkey_algo, use )) )
+          else if ( !(rc=openpgp_pk_test_algo2 (pk->pubkey_algo, use)) ) 
             {
               /* Skip the actual key if the key is already present
                * in the list.  Add it to our list if not. */
               if (key_present_in_pk_list(pk_list, pk) == 0)
                 {
                   free_public_key (pk); pk = NULL;
-                  if (!opt.quiet)
-                    log_info (_("%s: skipped: public key already present\n"),
-                              rov->d);
+                  log_info (_("%s: skipped: public key already present\n"),
+                            rov->d);
                 }
               else
                 {
@@ -863,13 +867,13 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
                     }
                 }
             }
-          else
+          else 
             {
               /* The public key is not usable for encryption or not
                  available. */
               free_public_key( pk ); pk = NULL;
               log_error(_("%s: skipped: %s\n"), rov->d, g10_errstr(rc) );
-              write_status_text_and_buffer (STATUS_INV_RECP, "3 ",
+              write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
                                             rov->d, strlen (rov->d), -1);
               goto fail;
             }
@@ -878,11 +882,11 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
 
   /* If we don't have any recipients yet and we are not in batch mode
      drop into interactive selection mode. */
-  if ( !any_recipients && !opt.batch )
-    {
+  if ( !any_recipients && !opt.batch ) 
+    { 
       int have_def_rec;
       char *answer = NULL;
-      STRLIST backlog = NULL;
+      strlist_t backlog = NULL;
 
       if (pk_list)
         any_recipients = 1;
@@ -891,7 +895,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
       if ( !have_def_rec )
         tty_printf(_("You did not specify a user ID. (you may use \"-r\")\n"));
 
-      for (;;)
+      for (;;) 
         {
           rc = 0;
           xfree(answer);
@@ -901,10 +905,10 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
               answer = def_rec;
               def_rec = NULL;
             }
-          else if (backlog)
+          else if (backlog) 
             {
               /* This is part of our trick to expand and display groups. */
-              answer = pop_strlist (&backlog);
+              answer = strlist_pop (&backlog);
             }
           else
             {
@@ -944,15 +948,15 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
               trim_spaces(answer);
               cpr_kill_prompt();
             }
-
-          if ( !answer || !*answer )
+          
+          if ( !answer || !*answer ) 
             {
               xfree(answer);
               break;  /* No more recipients entered - get out of loop. */
             }
 
           /* Do group expand here too.  The trick here is to continue
-             the loop if any expansion occurred.  The code above will
+             the loop if any expansion occured.  The code above will
              then list all expanded keys. */
           if (expand_id(answer,&backlog,0))
             continue;
@@ -962,15 +966,15 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
             free_public_key (pk);
           pk = xmalloc_clear( sizeof *pk );
           pk->req_usage = use;
-          rc = get_pubkey_byname( pk, answer, NULL, NULL, 0 );
+          rc = get_pubkey_byname (NULL, pk, answer, NULL, NULL, 0, 0 );
           if (rc)
             tty_printf(_("No such user ID.\n"));
-          else if ( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) )
+          else if ( !(rc=openpgp_pk_test_algo2 (pk->pubkey_algo, use)) ) 
             {
               if ( have_def_rec )
                 {
                   /* No validation for a default recipient. */
-                  if (!key_present_in_pk_list(pk_list, pk))
+                  if (!key_present_in_pk_list(pk_list, pk)) 
                     {
                       free_public_key (pk); pk = NULL;
                       log_info (_("skipped: public key "
@@ -990,13 +994,13 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
               else
                 { /* Check validity of this key. */
                   int trustlevel;
-
+		    
                   trustlevel = get_validity (pk, pk->user_id);
-                  if ( (trustlevel & TRUST_FLAG_DISABLED) )
+                  if ( (trustlevel & TRUST_FLAG_DISABLED) ) 
                     {
                       tty_printf (_("Public key is disabled.\n") );
                     }
-                  else if ( do_we_trust_pre (pk, trustlevel) )
+                  else if ( do_we_trust_pre (pk, trustlevel) ) 
                     {
                       /* Skip the actual key if the key is already
                        * present in the list */
@@ -1028,18 +1032,18 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
           pk = NULL;
         }
     }
-  else if ( !any_recipients && (def_rec = default_recipient()) )
+  else if ( !any_recipients && (def_rec = default_recipient()) ) 
     {
       /* We are in batch mode and have only a default recipient. */
       pk = xmalloc_clear( sizeof *pk );
       pk->req_usage = use;
 
       /* The default recipient is allowed to be disabled; thus pass 1
-         as last argument. */
-      rc = get_pubkey_byname (pk, def_rec, NULL, NULL, 1);
+         as second last argument.  We also don't want an AKL. */
+      rc = get_pubkey_byname (NULL, pk, def_rec, NULL, NULL, 1, 1);
       if (rc)
         log_error(_("unknown default recipient \"%s\"\n"), def_rec );
-      else if ( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) )
+      else if ( !(rc=openpgp_pk_test_algo2(pk->pubkey_algo, use)) ) 
         {
           /* Mark any_recipients here since the default recipient
              would have been used if it wasn't already there.  It
@@ -1049,7 +1053,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
           if (!key_present_in_pk_list(pk_list, pk))
             log_info (_("skipped: public key already set "
                         "as default recipient\n"));
-          else
+          else 
             {
               PK_LIST r = xmalloc( sizeof *r );
               r->pk = pk; pk = NULL;
@@ -1065,51 +1069,47 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
         }
       xfree(def_rec); def_rec = NULL;
     }
-  else
+  else 
     {
       /* General case: Check all keys. */
       any_recipients = 0;
-      for (; remusr; remusr = remusr->next )
+      for (; remusr; remusr = remusr->next ) 
         {
           if ( (remusr->flags & 1) )
             continue; /* encrypt-to keys are already handled. */
 
           pk = xmalloc_clear( sizeof *pk );
           pk->req_usage = use;
-          if ( (rc = get_pubkey_byname( pk, remusr->d, NULL, NULL, 0 )) )
+          if ((rc = get_pubkey_byname (NULL, pk, remusr->d, NULL, NULL, 0, 0)))
             {
               /* Key not found or other error. */
               free_public_key( pk ); pk = NULL;
               log_error(_("%s: skipped: %s\n"), remusr->d, g10_errstr(rc) );
-              write_status_text_and_buffer (STATUS_INV_RECP,
-                                            (rc == G10ERR_NO_PUBKEY
-                                             || rc == G10ERR_NO_SECKEY)? "1 ":
-                                            (rc == G10ERR_INV_USER_ID)? "14 ":
-                                            "0 ",
+              write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
                                             remusr->d, strlen (remusr->d),
                                             -1);
               goto fail;
             }
-          else if ( !(rc=check_pubkey_algo2(pk->pubkey_algo, use )) )
+          else if ( !(rc=openpgp_pk_test_algo2(pk->pubkey_algo, use )) ) 
             {
               /* Key found and usable.  Check validity. */
               int trustlevel;
-
+              
               trustlevel = get_validity (pk, pk->user_id);
-              if ( (trustlevel & TRUST_FLAG_DISABLED) )
+              if ( (trustlevel & TRUST_FLAG_DISABLED) ) 
                 {
                   /*Key has been disabled. */
                   free_public_key(pk); pk = NULL;
                   log_info(_("%s: skipped: public key is disabled\n"),
                            remusr->d);
-                  write_status_text_and_buffer (STATUS_INV_RECP, "13 ",
+                  write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
                                                 remusr->d,
                                                 strlen (remusr->d),
                                                 -1);
                   rc=G10ERR_UNU_PUBKEY;
                   goto fail;
                 }
-              else if ( do_we_trust_pre( pk, trustlevel ) )
+              else if ( do_we_trust_pre( pk, trustlevel ) ) 
                 {
                   /* Note: do_we_trust may have changed the trustlevel */
 
@@ -1119,12 +1119,11 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
 
                   /* Skip the actual key if the key is already present
                    * in the list */
-                  if (!key_present_in_pk_list(pk_list, pk))
+                  if (!key_present_in_pk_list(pk_list, pk)) 
                     {
                       free_public_key(pk); pk = NULL;
-                      if (!opt.quiet)
-                        log_info(_("%s: skipped: public key already present\n"),
-                                 remusr->d);
+                      log_info(_("%s: skipped: public key already present\n"),
+                               remusr->d);
                     }
                   else
                     {
@@ -1151,7 +1150,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
             {
               /* Key found but not usable for us (e.g. sign-only key). */
               free_public_key( pk ); pk = NULL;
-              write_status_text_and_buffer (STATUS_INV_RECP, "3 ",
+              write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
                                             remusr->d,
                                             strlen (remusr->d),
                                             -1);
@@ -1160,14 +1159,14 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
             }
         }
     }
-
-  if ( !rc && !any_recipients )
+  
+  if ( !rc && !any_recipients ) 
     {
       log_error(_("no valid addressees\n"));
       write_status_text (STATUS_NO_RECP, "0");
       rc = G10ERR_NO_USER_ID;
     }
-
+  
  fail:
 
   if ( rc )
@@ -1198,7 +1197,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned int use )
    preference list, so I'm including it. -dms */
 
 int
-algo_available( preftype_t preftype, int algo, const union pref_hint *hint )
+algo_available( preftype_t preftype, int algo, const union pref_hint *hint)
 {
   if( preftype == PREFTYPE_SYM )
     {
@@ -1206,7 +1205,7 @@ algo_available( preftype_t preftype, int algo, const union pref_hint *hint )
 		  && algo != CIPHER_ALGO_3DES
 		  && algo != CIPHER_ALGO_CAST5))
 	return 0;
-
+      
       if(PGP7 && (algo != CIPHER_ALGO_IDEA
 		  && algo != CIPHER_ALGO_3DES
 		  && algo != CIPHER_ALGO_CAST5
@@ -1218,22 +1217,22 @@ algo_available( preftype_t preftype, int algo, const union pref_hint *hint )
 
       /* PGP8 supports all the ciphers we do.. */
 
-      return algo && !check_cipher_algo( algo );
+      return algo && !openpgp_cipher_test_algo ( algo );
     }
   else if( preftype == PREFTYPE_HASH )
     {
-      if(hint && hint->digest_length)
+      if (hint && hint->digest_length)
 	{
-	  if(hint->digest_length!=20 || opt.flags.dsa2)
+	  if (hint->digest_length!=20 || opt.flags.dsa2)
 	    {
 	      /* If --enable-dsa2 is set or the hash isn't 160 bits
 		 (which implies DSA2), then we'll accept a hash that
 		 is larger than we need.  Otherwise we won't accept
 		 any hash that isn't exactly the right size. */
-	      if(hint->digest_length > md_digest_length(algo))
+	      if (hint->digest_length > gcry_md_get_algo_dlen (algo))
 		return 0;
 	    }
-	  else if(hint->digest_length != md_digest_length(algo))
+	  else if (hint->digest_length != gcry_md_get_algo_dlen (algo))
 	    return 0;
 	}
 
@@ -1249,7 +1248,7 @@ algo_available( preftype_t preftype, int algo, const union pref_hint *hint )
 		  && algo != DIGEST_ALGO_SHA256))
 	return 0;
 
-      return algo && !check_digest_algo( algo );
+      return algo && !openpgp_md_test_algo (algo);
     }
   else if( preftype == PREFTYPE_ZIP )
     {
@@ -1276,8 +1275,9 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
   u32 bits[8];
   const prefitem_t *prefs;
   int result=-1,i;
-  u16 scores[256];
-
+  unsigned int best=-1;    
+  byte scores[256];
+    
   if( !pk_list )
     return -1;
 
@@ -1297,7 +1297,8 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
 	  /* IDEA is implicitly there for v3 keys with v3 selfsigs if
 	     --pgp2 mode is on.  This was a 2440 thing that was
 	     dropped from 4880 but is still relevant to GPG's 1991
-	     support.  */
+	     support.  All this doesn't mean IDEA is actually
+	     available, of course. */
 	  if(PGP2 && pkr->pk->version<4 && pkr->pk->selfsigversion<4)
 	    implicit=CIPHER_ALGO_IDEA;
 	  else
@@ -1338,13 +1339,7 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
 	    {
 	      if( prefs[i].type == preftype )
 		{
-		  /* Make sure all scores don't add up past 0xFFFF
-		     (and roll around) */
-		  if(rank+scores[prefs[i].value]<=0xFFFF)
-		    scores[prefs[i].value]+=rank;
-		  else
-		    scores[prefs[i].value]=0xFFFF;
-
+		  scores[prefs[i].value]+=rank;
 		  mask[prefs[i].value/32] |= 1<<(prefs[i].value%32);
 
 		  rank++;
@@ -1411,31 +1406,10 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
 
   if(result==-1)
     {
-      unsigned int best=-1;
-
       /* At this point, we have not selected an algorithm due to a
 	 special request or via personal prefs.  Pick the highest
 	 ranked algorithm (i.e. the one with the lowest score). */
 
-      if(preftype==PREFTYPE_HASH && scores[DIGEST_ALGO_MD5])
-	{
-	  /* "If you are building an authentication system, the recipient
-	     may specify a preferred signing algorithm. However, the
-	     signer would be foolish to use a weak algorithm simply
-	     because the recipient requests it." (RFC4880:14).  If any
-	     other hash algorithm is available, pretend that MD5 isn't.
-	     Note that if the user intentionally chose MD5 by putting it
-	     in their personal prefs, then we do what the user said (as we
-	     never reach this code). */
-
-	  for(i=DIGEST_ALGO_MD5+1;i<256;i++)
-	    if(scores[i])
-	      {
-		scores[DIGEST_ALGO_MD5]=0;
-		break;
-	      }
-	}
-
       for(i=0;i<256;i++)
 	{
 	  /* Note the '<' here.  This means in case of a tie, we will
@@ -1453,32 +1427,91 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype,
 	      result=i;
 	    }
 	}
+
+      /* "If you are building an authentication system, the recipient
+	 may specify a preferred signing algorithm. However, the
+	 signer would be foolish to use a weak algorithm simply
+	 because the recipient requests it." (RFC4880:14).  If we
+	 settle on MD5, and SHA1 is also available, use SHA1 instead.
+	 Note that if the user intentionally chose MD5 by putting it
+	 in their personal prefs, then we do what the user said (as we
+	 never reach this code). */
+      if(preftype==PREFTYPE_HASH && result==DIGEST_ALGO_MD5
+	 && (bits[0] & (1<<DIGEST_ALGO_SHA1)))
+	result=DIGEST_ALGO_SHA1;
     }
 
   return result;
 }
 
 /*
- * Select the MDC flag from the pk_list.  We can only use MDC if all recipients
- * support this feature
+ * Select the MDC flag from the pk_list.  We can only use MDC if all
+ * recipients support this feature.
  */
 int
 select_mdc_from_pklist (PK_LIST pk_list)
 {
-    PK_LIST pkr;
+  PK_LIST pkr;
 
-    if( !pk_list )
-	return 0;
-
-    for (pkr = pk_list; pkr; pkr = pkr->next) {
-        int mdc;
-
-        if (pkr->pk->user_id) /* selected by user ID */
-            mdc = pkr->pk->user_id->flags.mdc;
-        else
-            mdc = pkr->pk->mdc_feature;
-        if (!mdc)
-            return 0; /* at least one recipient does not support it */
+  if ( !pk_list )
+    return 0;
+  
+  for (pkr = pk_list; pkr; pkr = pkr->next) 
+    {
+      int mdc;
+      
+      if (pkr->pk->user_id) /* selected by user ID */
+        mdc = pkr->pk->user_id->flags.mdc;
+      else
+        mdc = pkr->pk->mdc_feature;
+      if (!mdc)
+        return 0;  /* At least one recipient does not support it. */
+    }
+  return 1; /* Can be used. */
+}
+
+
+/* Print a warning for all keys in PK_LIST missing the MDC feature. */
+void
+warn_missing_mdc_from_pklist (PK_LIST pk_list)
+{
+  PK_LIST pkr;
+  
+  for (pkr = pk_list; pkr; pkr = pkr->next) 
+    {
+      int mdc;
+
+      if (pkr->pk->user_id) /* selected by user ID */
+        mdc = pkr->pk->user_id->flags.mdc;
+      else
+        mdc = pkr->pk->mdc_feature;
+      if (!mdc)
+        log_info (_("Note: key %s has no %s feature\n"),
+                  keystr_from_pk (pkr->pk), "MDC");
+    }
+}
+
+void
+warn_missing_aes_from_pklist (PK_LIST pk_list)
+{
+  PK_LIST pkr;
+ 
+  for (pkr = pk_list; pkr; pkr = pkr->next) 
+    {
+      const prefitem_t *prefs;
+      int i;
+      int gotit = 0;
+
+      prefs = pkr->pk->user_id? pkr->pk->user_id->prefs : pkr->pk->prefs;
+      if (prefs)
+        {
+          for (i=0; !gotit && prefs[i].type; i++ )
+            if (prefs[i].type == PREFTYPE_SYM 
+                && prefs[i].value == CIPHER_ALGO_AES)
+              gotit++;
+	}
+      if (!gotit)
+        log_info (_("Note: key %s has no preference for %s\n"),
+                  keystr_from_pk (pkr->pk), "AES");
     }
-    return 1; /* can be used */
 }
