@@ -1,6 +1,6 @@
 /* trustdb.c
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
- *               2008, 2012 Free Software Foundation, Inc.
+ *               2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,17 +26,13 @@
 
 #ifndef DISABLE_REGEX
 #include <sys/types.h>
-#ifdef USE_INTERNAL_REGEX
-#include "_regex.h"
-#else
 #include <regex.h>
-#endif
 #endif /* !DISABLE_REGEX */
 
-#include "errors.h"
+#include "gpg.h"
+#include "status.h"
 #include "iobuf.h"
 #include "keydb.h"
-#include "memory.h"
 #include "util.h"
 #include "options.h"
 #include "packet.h"
@@ -48,7 +44,7 @@
 
 /*
  * A structure to store key identification as well as some stuff needed
- * for validation
+ * for validation 
  */
 struct key_item {
   struct key_item *next;
@@ -64,7 +60,7 @@ typedef struct key_item **KeyHashTable; /* see new_key_hash_table() */
 
 /*
  * Structure to keep track of keys, this is used as an array wherre
- * the item right after the last one has a keyblock set to NULL.
+ * the item right after the last one has a keyblock set to NULL. 
  * Maybe we can drop this thing and replace it by key_item
  */
 struct key_array {
@@ -77,7 +73,6 @@ static struct {
     int init;
     int level;
     char *dbname;
-    int no_trustdb;   /* Set if a trustdb file is not available.  */
 } trustdb_args;
 
 /* some globals */
@@ -97,7 +92,7 @@ static struct key_item *
 new_key_item (void)
 {
   struct key_item *k;
-
+  
   k = xmalloc_clear (sizeof *k);
   return k;
 }
@@ -119,11 +114,11 @@ release_key_items (struct key_item *k)
  * For fast keylook up we need a hash table.  Each byte of a KeyIDs
  * should be distributed equally over the 256 possible values (except
  * for v3 keyIDs but we consider them as not important here). So we
- * can just use 10 bits to index a table of 1024 key items.
+ * can just use 10 bits to index a table of 1024 key items. 
  * Possible optimization: Don not use key_items but other hash_table when the
- * duplicates lists gets too large.
+ * duplicates lists gets too large. 
  */
-static KeyHashTable
+static KeyHashTable 
 new_key_hash_table (void)
 {
   struct key_item **tbl;
@@ -144,7 +139,7 @@ release_key_hash_table (KeyHashTable tbl)
   xfree (tbl);
 }
 
-/*
+/* 
  * Returns: True if the keyID is in the given hash table
  */
 static int
@@ -169,7 +164,7 @@ add_key_hash_table (KeyHashTable tbl, u32 *kid)
   for (k = tbl[(kid[1] & 0x03ff)]; k; k = k->next)
     if (k->kid[0] == kid[0] && k->kid[1] == kid[1])
       return; /* already in table */
-
+  
   kk = new_key_item ();
   kk->kid[0] = kid[0];
   kk->kid[1] = kid[1];
@@ -239,7 +234,7 @@ add_utk (u32 *kid)
 {
   struct key_item *k;
 
-  for (k = utk_list; k; k = k->next)
+  for (k = utk_list; k; k = k->next) 
     {
       if (k->kid[0] == kid[0] && k->kid[1] == kid[1])
         {
@@ -274,15 +269,15 @@ verify_own_keys(void)
     return;
 
   /* scan the trustdb to find all ultimately trusted keys */
-  for (recnum=1; !tdbio_read_record (recnum, &rec, 0); recnum++ )
+  for (recnum=1; !tdbio_read_record (recnum, &rec, 0); recnum++ ) 
     {
-      if ( rec.rectype == RECTYPE_TRUST
+      if ( rec.rectype == RECTYPE_TRUST 
            && (rec.r.trust.ownertrust & TRUST_MASK) == TRUST_ULTIMATE)
         {
             byte *fpr = rec.r.trust.fingerprint;
             int fprlen;
             u32 kid[2];
-
+            
             /* Problem: We do only use fingerprints in the trustdb but
              * we need the keyID here to indetify the key; we can only
              * use that ugly hack to distinguish between 16 and 20
@@ -298,9 +293,9 @@ verify_own_keys(void)
     }
 
   /* Put any --trusted-key keys into the trustdb */
-  for (k = user_utk_list; k; k = k->next)
+  for (k = user_utk_list; k; k = k->next) 
     {
-      if ( add_utk (k->kid) )
+      if ( add_utk (k->kid) ) 
         { /* not yet in trustDB as ultimately trusted */
           PKT_public_key pk;
 
@@ -446,7 +441,7 @@ init_trustdb()
 
   if(level==0 || level==1)
     {
-      int rc = tdbio_set_dbname (dbname, !!level, &trustdb_args.no_trustdb);
+      int rc = tdbio_set_dbname( dbname, !!level );
       if( rc )
 	log_fatal("can't init trustdb: %s\n", g10_errstr(rc) );
     }
@@ -497,7 +492,7 @@ init_trustdb()
 static int
 trust_letter (unsigned int value)
 {
-  switch( (value & TRUST_MASK) )
+  switch( (value & TRUST_MASK) ) 
     {
     case TRUST_UNKNOWN:   return '-';
     case TRUST_EXPIRED:   return 'e';
@@ -510,11 +505,7 @@ trust_letter (unsigned int value)
     }
 }
 
-const char *
-uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid)
-{
-  if(!key && !uid)
-/* TRANSLATORS: these strings are similar to those in
+/* NOTE TO TRANSLATOR: these strings are similar to those in
    trust_value_to_string(), but are a fixed length.  This is needed to
    make attractive information listings where columns line up
    properly.  The value "10" should be the length of the strings you
@@ -522,6 +513,10 @@ uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid)
    It gets passed to atoi() so everything after the number is
    essentially a comment and need not be translated.  Either key and
    uid are both NULL, or neither are NULL. */
+const char *
+uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid)
+{
+  if(!key && !uid)
     return _("10 translator see trustdb.c:uid_trust_string_fixed");
   else if(uid->is_revoked || (key && key->is_revoked))
     return                         _("[ revoked]");
@@ -546,7 +541,7 @@ uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid)
 const char *
 trust_value_to_string (unsigned int value)
 {
-  switch( (value & TRUST_MASK) )
+  switch( (value & TRUST_MASK) ) 
     {
     case TRUST_UNKNOWN:   return _("unknown");
     case TRUST_EXPIRED:   return _("expired");
@@ -615,7 +610,7 @@ check_trustdb ()
 
 
 /*
- * Recreate the WoT.
+ * Recreate the WoT. 
  */
 void
 update_trustdb()
@@ -632,9 +627,6 @@ void
 revalidation_mark (void)
 {
   init_trustdb();
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return;
-
   /* we simply set the time for the next check to 1 (far back in 1970)
    * so that a --update-trustdb will be scheduled */
   if (tdbio_write_nextcheck (1))
@@ -664,16 +656,13 @@ trustdb_check_or_update(void)
 
 void
 read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck,
-		   byte *marginals,byte *completes,byte *cert_depth,
-		   byte *min_cert_level)
+		   byte *marginals,byte *completes,byte *cert_depth)
 {
   TRUSTREC opts;
 
   init_trustdb();
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    memset (&opts, 0, sizeof opts);
-  else
-    read_record(0,&opts,RECTYPE_VER);
+
+  read_record(0,&opts,RECTYPE_VER);
 
   if(trust_model)
     *trust_model=opts.r.ver.trust_model;
@@ -687,37 +676,35 @@ read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck,
     *completes=opts.r.ver.completes;
   if(cert_depth)
     *cert_depth=opts.r.ver.cert_depth;
-  if(min_cert_level)
-    *min_cert_level=opts.r.ver.min_cert_level;
 }
 
 /***********************************************
  ***********  Ownertrust et al. ****************
  ***********************************************/
 
-static int
+static int 
 read_trust_record (PKT_public_key *pk, TRUSTREC *rec)
 {
   int rc;
-
+  
   init_trustdb();
   rc = tdbio_search_trust_bypk (pk, rec);
   if (rc == -1)
     return -1; /* no record yet */
-  if (rc)
+  if (rc) 
     {
       log_error ("trustdb: searching trust record failed: %s\n",
                  g10_errstr (rc));
-      return rc;
+      return rc; 
     }
-
+      
   if (rec->rectype != RECTYPE_TRUST)
     {
       log_error ("trustdb: record %lu is not a trust record\n",
                  rec->recnum);
-      return G10ERR_TRUSTDB;
-    }
-
+      return G10ERR_TRUSTDB; 
+    }      
+  
   return 0;
 }
 
@@ -725,19 +712,16 @@ read_trust_record (PKT_public_key *pk, TRUSTREC *rec)
  * Return the assigned ownertrust value for the given public key.
  * The key should be the primary key.
  */
-unsigned int
+unsigned int 
 get_ownertrust ( PKT_public_key *pk)
 {
   TRUSTREC rec;
   int rc;
-
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return TRUST_UNKNOWN;
-
+  
   rc = read_trust_record (pk, &rec);
   if (rc == -1)
     return TRUST_UNKNOWN; /* no record yet */
-  if (rc)
+  if (rc) 
     {
       tdbio_invalid ();
       return rc; /* actually never reached */
@@ -746,19 +730,16 @@ get_ownertrust ( PKT_public_key *pk)
   return rec.r.trust.ownertrust;
 }
 
-unsigned int
+unsigned int 
 get_min_ownertrust (PKT_public_key *pk)
 {
   TRUSTREC rec;
   int rc;
-
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return TRUST_UNKNOWN;
-
+  
   rc = read_trust_record (pk, &rec);
   if (rc == -1)
     return TRUST_UNKNOWN; /* no record yet */
-  if (rc)
+  if (rc) 
     {
       tdbio_invalid ();
       return rc; /* actually never reached */
@@ -821,10 +802,7 @@ update_ownertrust (PKT_public_key *pk, unsigned int new_trust )
 {
   TRUSTREC rec;
   int rc;
-
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return;
-
+  
   rc = read_trust_record (pk, &rec);
   if (!rc)
     {
@@ -856,7 +834,7 @@ update_ownertrust (PKT_public_key *pk, unsigned int new_trust )
       do_sync ();
       rc = 0;
     }
-  else
+  else 
     {
       tdbio_invalid ();
     }
@@ -869,9 +847,6 @@ update_min_ownertrust (u32 *kid, unsigned int new_trust )
   TRUSTREC rec;
   int rc;
 
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return;
-
   pk = xmalloc_clear (sizeof *pk);
   rc = get_pubkey (pk, kid);
   if (rc)
@@ -913,7 +888,7 @@ update_min_ownertrust (u32 *kid, unsigned int new_trust )
       do_sync ();
       rc = 0;
     }
-  else
+  else 
     {
       tdbio_invalid ();
     }
@@ -926,11 +901,7 @@ clear_ownertrusts (PKT_public_key *pk)
 {
   TRUSTREC rec;
   int rc;
-
-  init_trustdb ();
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return 0;
-
+  
   rc = read_trust_record (pk, &rec);
   if (!rc)
     {
@@ -958,8 +929,8 @@ clear_ownertrusts (PKT_public_key *pk)
   return 0;
 }
 
-/*
- * Note: Caller has to do a sync
+/* 
+ * Note: Caller has to do a sync 
  */
 static void
 update_validity (PKT_public_key *pk, PKT_user_id *uid,
@@ -978,7 +949,7 @@ update_validity (PKT_public_key *pk, PKT_user_id *uid,
       return;
     }
   if (rc == -1) /* no record yet - create a new one */
-    {
+    { 
       size_t dummy;
 
       rc = 0;
@@ -1033,8 +1004,6 @@ cache_disabled_value(PKT_public_key *pk)
     return (pk->is_disabled==2);
 
   init_trustdb();
-  if (trustdb_args.no_trustdb)
-    return 0;  /* No trustdb => not disabled.  */
 
   rc = read_trust_record (pk, &trec);
   if (rc && rc != -1)
@@ -1044,10 +1013,10 @@ cache_disabled_value(PKT_public_key *pk)
     }
   if (rc == -1) /* no record found, so assume not disabled */
     goto leave;
-
+ 
   if(trec.r.trust.ownertrust & TRUST_FLAG_DISABLED)
     disabled=1;
-
+ 
   /* Cache it for later so we don't need to look at the trustdb every
      time */
   if(disabled)
@@ -1065,9 +1034,6 @@ check_trustdb_stale(void)
   static int did_nextcheck=0;
 
   init_trustdb ();
-  if (trustdb_args.no_trustdb)
-    return;  /* No trustdb => can't be stale.  */
-
   if (!did_nextcheck
       && (opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC))
     {
@@ -1075,10 +1041,9 @@ check_trustdb_stale(void)
 
       did_nextcheck = 1;
       scheduled = tdbio_read_nextcheck ();
-      if ((scheduled && scheduled <= make_timestamp ())
-	  || pending_check_trustdb)
+      if (scheduled && scheduled <= make_timestamp ())
         {
-          if (opt.no_auto_check_trustdb)
+          if (opt.no_auto_check_trustdb) 
             {
               pending_check_trustdb = 1;
               log_info (_("please do a --check-trustdb\n"));
@@ -1095,7 +1060,7 @@ check_trustdb_stale(void)
 /*
  * Return the validity information for PK.  If the namehash is not
  * NULL, the validity of the corresponsing user ID is returned,
- * otherwise, a reasonable value for the entire key is returned.
+ * otherwise, a reasonable value for the entire key is returned. 
  */
 unsigned int
 get_validity (PKT_public_key *pk, PKT_user_id *uid)
@@ -1111,14 +1076,6 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
     namehash_from_uid(uid);
 
   init_trustdb ();
-
-  /* If we have no trustdb (which also means it has not been created)
-     and the trust-model is always, we don't know the validity -
-     return immediately.  If we won't do that the tdbio code would try
-     to open the trustdb and run into a fatal error.  */
-  if (trustdb_args.no_trustdb && opt.trust_model == TM_ALWAYS)
-    return TRUST_UNKNOWN;
-
   check_trustdb_stale();
 
   keyid_from_pk (pk, kid);
@@ -1132,7 +1089,7 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
           log_error ("error getting main key %s of subkey %s: %s\n",
                      tempkeystr, keystr(kid), g10_errstr(rc));
 	  xfree(tempkeystr);
-          validity = TRUST_UNKNOWN;
+          validity = TRUST_UNKNOWN; 
           goto leave;
 	}
     }
@@ -1155,7 +1112,7 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
     }
   if (rc == -1) /* no record found */
     {
-      validity = TRUST_UNKNOWN;
+      validity = TRUST_UNKNOWN; 
       goto leave;
     }
 
@@ -1188,7 +1145,7 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
 
       recno = vrec.r.valid.next;
     }
-
+  
   if ( (trec.r.trust.ownertrust & TRUST_FLAG_DISABLED) )
     {
       validity |= TRUST_FLAG_DISABLED;
@@ -1207,7 +1164,7 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
    * I initially designed it that way */
   if (main_pk->has_expired || pk->has_expired)
     validity = (validity & ~TRUST_MASK) | TRUST_EXPIRED;
-
+  
   if (pending_check_trustdb)
     validity |= TRUST_FLAG_PENDING_CHECK;
 
@@ -1219,15 +1176,15 @@ get_validity (PKT_public_key *pk, PKT_user_id *uid)
 int
 get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
-    int trustlevel;
-
-    if (!pk)
-      return '?';  /* Just in case a NULL PK is passed.  */
-
-    trustlevel = get_validity (pk, uid);
-    if( trustlevel & TRUST_FLAG_REVOKED )
-	return 'r';
-    return trust_letter ( trustlevel );
+  int trustlevel;
+  
+  if (!pk)
+    return '?';  /* Just in case a NULL PK is passed.  */
+  
+  trustlevel = get_validity (pk, uid);
+  if ( (trustlevel & TRUST_FLAG_REVOKED) )
+    return 'r';
+  return trust_letter (trustlevel);
 }
 
 const char *
@@ -1283,6 +1240,7 @@ get_validity_counts (PKT_public_key *pk, PKT_user_id *uid)
 void
 list_trust_path( const char *username )
 {
+  (void)username;
 }
 
 /****************
@@ -1307,7 +1265,11 @@ int
 enum_cert_paths( void **context, ulong *lid,
 		 unsigned *ownertrust, unsigned *validity )
 {
-    return -1;
+  (void)context;
+  (void)lid;
+  (void)ownertrust;
+  (void)validity;
+  return -1;
 }
 
 
@@ -1315,10 +1277,13 @@ enum_cert_paths( void **context, ulong *lid,
  * Print the current path
  */
 void
-enum_cert_paths_print( void **context, FILE *fp,
-				       int refresh, ulong selected_lid )
+enum_cert_paths_print (void **context, FILE *fp,
+                       int refresh, ulong selected_lid)
 {
-    return;
+  (void)context;
+  (void)fp;
+  (void)refresh;
+  (void)selected_lid;
 }
 
 
@@ -1342,7 +1307,7 @@ ask_ownertrust (u32 *kid,int minimum)
                  keystr(kid), g10_errstr(rc) );
       return TRUST_UNKNOWN;
     }
-
+ 
   if(opt.force_ownertrust)
     {
       log_info("force trust for key %s to %s\n",
@@ -1415,7 +1380,7 @@ dump_key_array (int depth, struct key_array *keys)
             }
         }
     }
-}
+}  
 
 
 static void
@@ -1438,7 +1403,7 @@ store_validation_status (int depth, KBNODE keyblock, KeyHashTable stored)
             status = TRUST_UNDEFINED;
           else
             status = 0;
-
+          
           if (status)
             {
               update_validity (keyblock->pkt->pkt.public_key,
@@ -1453,7 +1418,7 @@ store_validation_status (int depth, KBNODE keyblock, KeyHashTable stored)
 
   if (any)
     do_sync ();
-}
+}  
 
 /*
  * check whether the signature sig is in the klist k
@@ -1485,7 +1450,7 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
 {
   KBNODE node;
   PKT_signature *sig;
-
+  
   /* first check all signatures */
   for (node=uidnode->next; node; node = node->next)
     {
@@ -1518,7 +1483,7 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
 	  continue;
 	}
       node->flag |= 1<<9;
-    }
+    }      
   /* reset the remaining flags */
   for (; node; node = node->next)
       node->flag &= ~(1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12);
@@ -1566,7 +1531,7 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
              older: if signode was older then we don't want to take n
              as signode is nonrevocable.  If n was older then we're
              automatically fine. */
-
+	  
 	  if(((IS_UID_SIG(signode->pkt->pkt.signature) &&
 	       !signode->pkt->pkt.signature->flags.revocable &&
 	       (signode->pkt->pkt.signature->expiredate==0 ||
@@ -1582,7 +1547,7 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
              n was older then we don't want to take signode as n is
              nonrevocable.  If signode was older then we're
              automatically fine. */
-
+	  
 	  if((!(IS_UID_SIG(signode->pkt->pkt.signature) &&
 		!signode->pkt->pkt.signature->flags.revocable &&
 		(signode->pkt->pkt.signature->expiredate==0 ||
@@ -1613,7 +1578,7 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
 
       sig = signode->pkt->pkt.signature;
       if (IS_UID_SIG (sig))
-        { /* this seems to be a usable one which is not revoked.
+        { /* this seems to be a usable one which is not revoked. 
            * Just need to check whether there is an expiration time,
            * We do the expired certification after finding a suitable
            * certification, the assumption is that a signator does not
@@ -1622,9 +1587,9 @@ mark_usable_uid_certs (KBNODE keyblock, KBNODE uidnode,
            * different expiration time */
           const byte *p;
           u32 expire;
-
+                    
           p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_SIG_EXPIRE, NULL );
-          expire = p? sig->timestamp + buf32_to_u32 (p) : 0;
+          expire = p? sig->timestamp + buffer_to_u32(p) : 0;
 
           if (expire==0 || expire > curtime )
             {
@@ -1709,7 +1674,7 @@ clean_sigs_from_uid(KBNODE keyblock,KBNODE uidnode,int noisy,int self_only)
       delete_kbnode(node);
       deleted++;
     }
-
+    
   return deleted;
 }
 
@@ -1818,10 +1783,6 @@ clean_key(KBNODE keyblock,int noisy,int self_only,
 /* Returns a sanitized copy of the regexp (which might be "", but not
    NULL). */
 #ifndef DISABLE_REGEX
-/* Operator charactors except '.' and backslash.
-   See regex(7) on BSD.  */
-#define REGEXP_OPERATOR_CHARS "^[$()|*+?{"
-
 static char *
 sanitize_regexp(const char *old)
 {
@@ -1861,7 +1822,7 @@ sanitize_regexp(const char *old)
     {
       if(!escaped && old[start]=='\\')
 	escaped=1;
-      else if (!escaped && strchr (REGEXP_OPERATOR_CHARS, old[start]))
+      else if(!escaped && old[start]!='.')
 	new[idx++]='\\';
       else
 	escaped=0;
@@ -1972,7 +1933,7 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
             {
               if (uid->help_full_count >= opt.completes_needed
                   || uid->help_marginal_count >= opt.marginals_needed )
-                uidnode->flag |= 4;
+                uidnode->flag |= 4; 
               else if (uid->help_full_count || uid->help_marginal_count)
                 uidnode->flag |= 2;
               uidnode->flag |= 1;
@@ -1987,7 +1948,7 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 
           issigned = 0;
 	  get_validity_counts(pk,uid);
-          mark_usable_uid_certs (kb, uidnode, main_kid, klist,
+          mark_usable_uid_certs (kb, uidnode, main_kid, klist, 
                                  curtime, next_expire);
         }
       else if (node->pkt->pkttype == PKT_SIGNATURE
@@ -1995,26 +1956,28 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
         {
 	  /* Note that we are only seeing unrevoked sigs here */
           PKT_signature *sig = node->pkt->pkt.signature;
-
+          
           kr = is_in_klist (klist, sig);
 	  /* If the trust_regexp does not match, it's as if the sig
              did not exist.  This is safe for non-trust sigs as well
              since we don't accept a regexp on the sig unless it's a
              trust sig. */
-          if (kr && (kr->trust_regexp==NULL || opt.trust_model!=TM_PGP ||
-		     (uidnode && check_regexp(kr->trust_regexp,
-					    uidnode->pkt->pkt.user_id->name))))
+          if (kr && (!kr->trust_regexp 
+                     || opt.trust_model != TM_PGP 
+                     || (uidnode 
+                         && check_regexp(kr->trust_regexp,
+                                         uidnode->pkt->pkt.user_id->name))))
             {
 	      /* Are we part of a trust sig chain?  We always favor
                  the latest trust sig, rather than the greater or
                  lesser trust sig or value.  I could make a decent
                  argument for any of these cases, but this seems to be
                  what PGP does, and I'd like to be compatible. -dms */
-	      if(opt.trust_model==TM_PGP
-		 && sig->trust_depth
-		 && pk->trust_timestamp<=sig->timestamp)
+              if (opt.trust_model == TM_PGP
+                  && sig->trust_depth
+                  && pk->trust_timestamp <= sig->timestamp)
 		{
-		  byte depth;
+		  unsigned char depth;
 
 		  /* If the depth on the signature is less than the
 		     chain currently has, then use the signature depth
@@ -2026,20 +1989,20 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 		     trusted signature can restart the depth to
 		     whatever level it likes. */
 
-		  if(sig->trust_depth<kr->trust_depth
-		     || kr->ownertrust==TRUST_ULTIMATE)
-		    depth=sig->trust_depth;
+		  if (sig->trust_depth < kr->trust_depth
+                      || kr->ownertrust == TRUST_ULTIMATE)
+		    depth = sig->trust_depth;
 		  else
-		    depth=kr->trust_depth;
+		    depth = kr->trust_depth;
 
-		  if(depth)
+		  if (depth)
 		    {
 		      if(DBG_TRUST)
-			log_debug("trust sig on %s, sig depth is %d,"
-				  " kr depth is %d\n",
-				  uidnode->pkt->pkt.user_id->name,
-				  sig->trust_depth,
-				  kr->trust_depth);
+			log_debug ("trust sig on %s, sig depth is %d,"
+                                   " kr depth is %d\n",
+                                   uidnode->pkt->pkt.user_id->name,
+                                   sig->trust_depth,
+                                   kr->trust_depth);
 
 		      /* If we got here, we know that:
 
@@ -2057,19 +2020,19 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 			 successfully.
 		      */
 
-		      if(DBG_TRUST)
-			log_debug("replacing trust value %d with %d and "
-				  "depth %d with %d\n",
-				  pk->trust_value,sig->trust_value,
-				  pk->trust_depth,depth);
-
-		      pk->trust_value=sig->trust_value;
-		      pk->trust_depth=depth-1;
+		      if (DBG_TRUST)
+			log_debug ("replacing trust value %d with %d and "
+                                   "depth %d with %d\n",
+                                   pk->trust_value,sig->trust_value,
+                                   pk->trust_depth,depth);
 
+		      pk->trust_value = sig->trust_value;
+		      pk->trust_depth = depth-1;
+                      
 		      /* If the trust sig contains a regexp, record it
 			 on the pk for the next round. */
-		      if(sig->trust_regexp)
-			pk->trust_regexp=sig->trust_regexp;
+		      if (sig->trust_regexp)
+			pk->trust_regexp = sig->trust_regexp;
 		    }
 		}
 
@@ -2088,7 +2051,7 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
     {
       if (uid->help_full_count >= opt.completes_needed
 	  || uid->help_marginal_count >= opt.marginals_needed )
-        uidnode->flag |= 4;
+        uidnode->flag |= 4; 
       else if (uid->help_full_count || uid->help_marginal_count)
         uidnode->flag |= 2;
       uidnode->flag |= 1;
@@ -2102,6 +2065,7 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 static int
 search_skipfnc (void *opaque, u32 *kid, PKT_user_id *dummy)
 {
+  (void)dummy;
   return test_key_hash_table ((KeyHashTable)opaque, kid);
 }
 
@@ -2111,7 +2075,7 @@ search_skipfnc (void *opaque, u32 *kid, PKT_user_id *dummy)
  * kllist.  The caller has to pass keydb handle so that we don't use
  * to create our own.  Returns either a key_array or NULL in case of
  * an error.  No results found are indicated by an empty array.
- * Caller hast to release the returned array.
+ * Caller hast to release the returned array.  
  */
 static struct key_array *
 validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
@@ -2122,11 +2086,11 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
   size_t nkeys, maxkeys;
   int rc;
   KEYDB_SEARCH_DESC desc;
-
+  
   maxkeys = 1000;
   keys = xmalloc ((maxkeys+1) * sizeof *keys);
   nkeys = 0;
-
+  
   rc = keydb_search_reset (hd);
   if (rc)
     {
@@ -2151,21 +2115,21 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
       xfree (keys);
       return NULL;
     }
-
+  
   desc.mode = KEYDB_SEARCH_MODE_NEXT; /* change mode */
   do
     {
       PKT_public_key *pk;
-
+        
       rc = keydb_get_keyblock (hd, &keyblock);
-      if (rc)
+      if (rc) 
         {
           log_error ("keydb_get_keyblock failed: %s\n", g10_errstr(rc));
           xfree (keys);
           return NULL;
         }
-
-      if ( keyblock->pkt->pkttype != PKT_PUBLIC_KEY)
+      
+      if ( keyblock->pkt->pkttype != PKT_PUBLIC_KEY) 
         {
           log_debug ("ooops: invalid pkttype %d encountered\n",
                      keyblock->pkt->pkttype);
@@ -2175,7 +2139,7 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
         }
 
       /* prepare the keyblock for further processing */
-      merge_keys_and_selfsig (keyblock);
+      merge_keys_and_selfsig (keyblock); 
       clear_kbnode_flags (keyblock);
       pk = keyblock->pkt->pkt.public_key;
       if (pk->has_expired || pk->is_revoked)
@@ -2212,9 +2176,9 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
 
       release_kbnode (keyblock);
       keyblock = NULL;
-    }
+    } 
   while ( !(rc = keydb_search (hd, &desc, 1)) );
-  if (rc && rc != -1)
+  if (rc && rc != -1) 
     {
       log_error ("keydb_search_next failed: %s\n", g10_errstr(rc));
       xfree (keys);
@@ -2223,7 +2187,7 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
 
   keys[nkeys].keyblock = NULL;
   return keys;
-}
+} 
 
 /* Caller must sync */
 static void
@@ -2233,7 +2197,7 @@ reset_trust_records(void)
   ulong recnum;
   int count = 0, nreset = 0;
 
-  for (recnum=1; !tdbio_read_record (recnum, &rec, 0); recnum++ )
+  for (recnum=1; !tdbio_read_record (recnum, &rec, 0); recnum++ ) 
     {
       if(rec.rectype==RECTYPE_TRUST)
 	{
@@ -2272,7 +2236,7 @@ reset_trust_records(void)
  * Step 2: loop max_cert_times
  * Step 3:   if OWNERTRUST of any key in klist is undefined
  *             ask user to assign ownertrust
- * Step 4:   Loop over all keys in the keyDB which are not marked seen
+ * Step 4:   Loop over all keys in the keyDB which are not marked seen 
  * Step 5:     if key is revoked or expired
  *                mark key as seen
  *                continue loop at Step 4
@@ -2284,7 +2248,7 @@ reset_trust_records(void)
  *             End Loop
  * Step 8:   Build a new klist from all fully trusted keys from step 6
  *           End Loop
- *         Ready
+ *         Ready  
  *
  */
 static int
@@ -2354,7 +2318,7 @@ validate_keys (int interactive)
       if ( pk->expiredate && pk->expiredate >= start_time
            && pk->expiredate < next_expire)
         next_expire = pk->expiredate;
-
+      
       release_kbnode (keyblock);
       do_sync ();
     }
@@ -2388,7 +2352,7 @@ validate_keys (int interactive)
 	    {
 	      k->ownertrust = ask_ownertrust (k->kid,min);
 
-	      if (k->ownertrust == (unsigned int)(-1))
+	      if (k->ownertrust == -1)
 		{
 		  quit=1;
 		  goto leave;
@@ -2430,7 +2394,7 @@ validate_keys (int interactive)
       /* Find all keys which are signed by a key in kdlist */
       keys = validate_key_list (kdb, full_trust, klist,
 				start_time, &next_expire);
-      if (!keys)
+      if (!keys) 
         {
           log_error ("validate_key_list failed\n");
           rc = G10ERR_GENERAL;
@@ -2441,16 +2405,16 @@ validate_keys (int interactive)
         ;
 
       /* Store the calculated valididation status somewhere */
-      if (opt.verbose > 1 && DBG_TRUST)
+      if (opt.verbose > 1)
         dump_key_array (depth, keys);
 
       for (kar=keys; kar->keyblock; kar++)
           store_validation_status (depth, kar->keyblock, stored);
 
       log_info (_("depth: %d  valid: %3d  signed: %3d"
-                  "  trust: %d-, %dq, %dn, %dm, %df, %du\n"),
+                  "  trust: %d-, %dq, %dn, %dm, %df, %du\n"), 
                 depth, valids, key_count, ot_unknown, ot_undefined,
-                ot_never, ot_marginal, ot_full, ot_ultimate );
+                ot_never, ot_marginal, ot_full, ot_ultimate ); 
 
       /* Build a new kdlist from all fully valid keys in KEYS */
       if (klist != utk_list)
@@ -2512,10 +2476,10 @@ validate_keys (int interactive)
   if (!rc && !quit) /* mark trustDB as checked */
     {
       if (next_expire == 0xffffffff || next_expire < start_time )
-        tdbio_write_nextcheck (0);
+        tdbio_write_nextcheck (0); 
       else
         {
-          tdbio_write_nextcheck (next_expire);
+          tdbio_write_nextcheck (next_expire); 
           log_info (_("next trustdb check due at %s\n"),
                     strtimestamp (next_expire));
         }
